#!/usr/local/bin/perl
#
# OMICHAIN
#
# XMM Pipeline Processing System OMICHAIN script
#
# Author: Jon Rainger (jfr@mssl.ucl.ac.uk)
#
# Description: This script runs the tasks required for to produce
#              PPS products from Imaging Mode ODFs.
#
# $Id: module.f90,v 1.6 1998/05/27 13:45:39 gvacanti Exp $
#
#
#    MODIFICATIONS:
#    12/06/2000 - AMK - Removed call to OMPSF as none of the pipleline tasks use it's output.
#
#    12/06/2000 - AMK - Redirected sreen output to a file omichain_log as user will lose the output during a complete ODF
#
#    06/09/2000 - AMK - Ammended code so that the new style house keeping files are processed correctly.
#
#    06/09/2000 - AMK - Ammended code so that product names conform to specification.
#
#    29/09/2000 - AMk - Changed flat field product file name to tmp_flat_field as this file should NOT be a product.
#
#    11/10/2000 - AMK - Removed the die calls and replace with SAS::fatal calls.
#
#    03/11/2000 - CB  - Modified so that images with "BLOCKED" or "UNKNOWN" filters are not processed.
#                       Added call to ps2pdf.
#    21/01/2000 - CB - Modified so that "ommosaic" runs at the end to produce a composite image for each filter.
#************************************************************************************************************************
use DAL;
use SAS;
#
sub omichain {

#////////////////////////////////////////
#
# AMK - 05/07/2000
#
# Created two flags for consistency checks
#
#  

#    open (STDOUT, ">omslewchain_log");
#    open (STDERR, ">&STDOUT");
#    S| = 1;
#    select (STDERR);
# 

    $periodic_file_flag = 0;
    $non_periodic_file_flag = 0;
    $flatfield_file_flag = 0;
#
##########################################
    $taskname='OMICHAIN';
    $VERSION='2.0';
    $date=&date_time;
    $create = "Running SAS task $taskname V$VERSION $date";
    $oal_directory = stringParameter("oaldirectory");
    if ($oal_directory eq "") 
	{         
	 $oal_directory = ".\/";
         $cur_dir = `pwd`;
#         $ENV{SAS_ODF} = $cur_dir; 
#print("SAS_ODF =  $ENV{SAS_ODF}");  
#         print("oal directory=$oal_directory\n");
#	 print("SAS_ODF= $ENV{SAS_ODF}, \n");
	 
	}
    $inp_directory = stringParameter("inpdirectory");
    if ($inp_directory eq "")
        {
         $inp_directory = $oal_directory;
	}
    $out_directory = stringParameter("outdirectory");
    if ($out_directory eq "")
        {
         $out_directory = $inp_directory;
  	}

#    print("$create \n");
    SAS::message($SAS::AppMsg, $SAS::SparseMsg, $create);

#    print("oaldirectory = $oal_directory \n");
    $message = "oaldirectory = " . $oal_directory;
    SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


#    print("inpdirectory = $inp_directory \n");
#    print("outdirectory = $out_directory \n");
#************************************************************
# Get a list of file names in the current working directory.
#************************************************************
opendir(INDIR, $inp_directory) ||
                        SAS::fatal("OMICHAIN","Unable to open $inp_directory"); 

#******************************************************************************
# Sort the file names according to observation, exposure, window and file type.
#******************************************************************************
while ( $file = readdir(INDIR))
{
   if ($file =~ /^([0-9A-Z]{4})_([0-9A-Z]{10})_OM([[0-9a-zA-Z]{4})([0-9]{2})([A-Z]{3}).FIT/i)
   {
      if ($file =~ /^([0-9A-Z]{4})_([0-9A-Z]{10})_OM[S|U|X][0-9a-zA-Z]{3}[0-9]{2}FFX.FIT/i) 
      {
         $ffx_list{"$1$2"} = "$inp_directory/$file";
      }
      else
      {
#***************************************************************************************************
# We need to filter out the housekeeping files for this loop. There is now only one PEH and NPH file 
# per ODF.
#
#
# We are now picking up the exposure type flag and this stored with the exposure number.
# This is necessary as we are required to insert the exposure type flag into the product file names. 
# This necessitates some further filtering below.
#****************************************************************************************************
	 $in_orbit_flat_field_filename;
         $flatfield;  # For creating a flatfield file if necessary 
	 if ($file !~ /^([0-9A-Z]{4})_([0-9A-Z]{10})_OM([[0-9a-zA-Z]{4})([0-9]{2})PEH.FIT/i
	     && $file !~ /^([0-9A-Z]{4})_([0-9A-Z]{10})_OM([[0-9a-zA-Z]{4})([0-9]{2})NPH.FIT/i)
#             && $file !~ /^([0-9A-Z]{4})_([0-9A-Z]{10})_OMX00000FFX.FIT/i)
       
	 {
	     $flatfield = $file;

             if($file =~  /"FFX"/)  # Checking for a flatfield file
             {
		 $flatfield_filename = $file;
#                 print ("flat field exists- $file \n\n");
		 $flatfield_file_flag = 1;
                 
	     }



             #******************
             # Orbit number list
             #******************
	     $orb_list{$1} = "$1";
             #****************** 
             # Observation list.
             #******************
	     ${${"$1"}{$2}} = "$1$2";
             #**********************************************
             # Exposure list.
             # This contains a value like 'X123'. X is the exposure flag and 123 i the exposure number.
             #**********************************************
	     ${${"$1$2"}{$3}} = "$1$2$3";
             #*************
             # Window list.
             #************* 
	     ${${"$1$2$3"}{$4}} = "$1$2$3$4";
             #***********
             # File list.
             #***********
             ${${"$1$2$3$4"}{$5}} = "$file";
         }
         else
         {
         #*****************************************
         # Pick-up the House Keeping files here.
         #*****************************************
 
	     if ($file =~ /^([0-9A-Z]{4})_([0-9A-Z]{10})_OM([[0-9a-zA-Z]{4})([0-9]{2})PEH.FIT/i)
	     {
		 $periodic_hk_filename = $file;

		 $periodic_file_flag = 1;
	     }

	     if($file =~ /^([0-9A-Z]{4})_([0-9A-Z]{10})_OM([[0-9a-zA-Z]{4})([0-9]{2})NPH.FIT/i)
	     {
		 $non_periodic_hk_filename = $file;
		 $non_periodic_file_flag = 1;
	     }
             if($file =~  "FFX")  # Checking for a flatfield file
             {
		 $flatfield_filename = $file;
#                 print ("flat field exists- $file \n\n");
		 $flatfield_file_flag = 1;
	     }


         }
      }
   }
}

closedir(INDIR);



  
#**************************************************
# Check for the presence of the house keeping files
#**************************************************
   if($periodic_file_flag == 0)
   {
       $message = "The ODF does NOT contain a Periodic Housekeeping file. Please insert into the ODF.";
       SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

       SAS::fatal("Task Failure","Incomplete ODF.");
   }

   if($non_periodic_file_flag == 0)
   {
       $message = "The ODF does NOT contain a Non Periodic Housekeeping file. Please insert into the ODF.";
       SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

       SAS::fatal("Task Failure","incomplete ODF.");
   }

    @FlatFieldFiles = <*FFX.FIT*>;        
    $numFlatFieldFiles = @FlatFieldFiles;
       

   if($numFlatFieldFiles == 0)
   {
  
           
       $fileName = substr($flatfield, 0, 15)  . "_OMX00000FFX.FIT";
       $arg_list = "outset=". $out_directory . $fileName;

       $message = "omflatgen $arg_list";
       SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

       system("omflatgen $arg_list") && SAS::fatal("Task Failure", "omflatgen");
       $ffx_list{"$1$2"} = "$inp_directory/$fileName";
       $in_orbit_flat_field_filename = $out_directory . $fileName;            
  
   }
   else
   {
        $in_orbit_flat_field_filename = @FlatFieldFiles[0];       # The SIMAGE2000 file name
   }   

#**********************************************
# Run the pipeline- Loop over all observations.
#**********************************************
   foreach $orb_key (keys(%orb_list))
   {
   $obs_list = $orb_list{$orb_key};

   $message = "processing Orbit $orb_key";
   SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

   foreach $obs_key (keys(%{$obs_list}))
      {$exp_list = ${${$obs_list}{$obs_key}};

      $message = "           Observation $obs_key \n";
      SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

#***************************
# Check consistancy of odf.
#***************************
#      if (exists($ffx_list{$orb_key . $obs_key}))
#         {
#	     $in_orbit_flat_field_filename =  $ffx_list{$orb_key . $obs_key };

	     $message = "Using flatfield $in_orbit_flat_field_filename \n";
             SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

#*************************************************************************
# Loop over all exposures- if an unused filter is found
# the exposure is not processed.
#*************************************************************************
         foreach $exposure_key (keys(%{$exp_list}))
            {

         #********************************************************************************************** 
         # We must take out the first character of the exposure key as it now contains the exposure flag
         #**********************************************************************************************
	     $exp_type = substr($exposure_key,0,1);
	     $exp_key = substr($exposure_key,1,3);
	
             $message = "Checking ORBIT $orb_key OBSERVATION $obs_key EXPOSURE $exp_key: \n";
             SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

             $message = "[THX][WDX][PROCESSING]";
             SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

             #**************************   
             # Check consistency of odf.
             #**************************
 
             if (exists(${$orb_key . $obs_key . $exp_type . $exp_key . "00"}{THX})) 
	        {
	        $message = " +";
                SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

		$tracking_history_filename = $inp_directory . "/". ${${$orb_key . $obs_key . $exp_type . $exp_key . "00"}{"THX"}};
                }	 
             else    
	        {
	        $message = " -";
                SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                $sas_dir = $ENV{SAS_DIR};
                #***********************************
                # Create a dummy name to fool OMPREP
                #***********************************
		$tracking_history_filename = "DUMMYTHX.FIT"; 
                }

             if (exists(${$orb_key . $obs_key . $exp_type . $exp_key . "00"}{WDX})) 
	        {
	       
                SAS::message($SAS::AppMsg, $SAS::SparseMsg, "+:");

                }	 
             else    
	        {
	        SAS::message($SAS::AppMsg, $SAS::SparseMsg, "-:");

                }
                #****************************
                # Check consistancy of odf.
                #
                # We need only check for the presence of the WDX and THX files here.
                #
                # We now have a true or dummy tracking history file name
                #****************************

             if(exists(${$orb_key . $obs_key . $exp_type . $exp_key . "00"}{WDX}))
               {
     
               SAS::message($SAS::AppMsg, $SAS::SparseMsg, "+ ");

               #**************************
               # Set pipeline variables.
               #**************************
               $window_data_filename = ${${$orb_key . $obs_key . $exp_type . $exp_key . "00"}{"WDX"}};


               $tracking_history_plot_filename = "P" . $obs_key . "OM" . $exp_type . $exp_key . "TSHPLT" . "0000.ps";

     
               $tracking_history_plot_filename1 = "P" . $obs_key . "OM" . $exp_type . $exp_key . "TSHPLT" . "0000.PDF";

               $tracking_history_timeseries_filename =  "P" . $obs_key . "OM" . $exp_type . $exp_key . "TSTRTS0000.FIT";
               #************************************
               # Produce tracking history products.
               #************************************
               $message = "... Running Tracking pipeline ...\n";
               SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

               #********************************************
               #  omprep
               #********************************************
               $message = "... omprep ...";
               SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

               $arg_list = "set=" . $tracking_history_filename .
                           " nphset=" . $inp_directory . '/' . $non_periodic_hk_filename .
                           " pehset=" . $inp_directory . '/' . $periodic_hk_filename .
                           " wdxset=" . $inp_directory . '/' . $window_data_filename .
                           " outset=" . $out_directory . '/' . "tmp_tracking" ;                           

	       system("omprep -v");
               $message = "... omprep  $arg_list\n";  
               SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

               system("omprep  $arg_list");

               $exit_value = $? >> 8;   # 0 okay, 1 call fatal called, or some other disaster
               if ($exit_value eq 1) # Failure in omprep 
               { 
                   $message = "omprep has detected an error- this observation will not be processed\n";
                   SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

               }     
               else
               {

               system("cp tmp_tracking omprep_thx.fit");

               #************************************************************
               # omdrifthist
               #************************************************************
               $message = "... omdrifthist ...\n";
               SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

               $arg_list = "set=" . $out_directory . '/' . "tmp_tracking" .
                           " plotfile=" . $out_directory . '/' . $tracking_history_plot_filename;

               $message = "omdrifthist $arg_list";
               SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

               system("omdrifthist $arg_list") && SAS::fatal("Task Failue","omdrifthist failed.");
	     

               #*******************************************************************
               # Only run ps2df if tracking-history file exists
               #*******************************************************************
               if (exists(${$orb_key . $obs_key . $exp_type . $exp_key . "00"}{THX})) 
	       {
                   $message = "ps2pdf ...";
                   SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                   system("ps2pdf  $tracking_history_plot_filename   $tracking_history_plot_filename1") && SAS::fatal("Task Failure","ps2pdf failed.");
     
               }

               #************************************************************
               # omthconv
               #************************************************************
               $message = "... omthconv ...\n";
               SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

               $arg_list = "thxset=" . $out_directory . '/' . "tmp_tracking" .
                           " nphset=" .  $inp_directory . '/' . $non_periodic_hk_filename .
                           " outset=" .  $out_directory . '/' . $tracking_history_timeseries_filename ;

               $nessage = "omthconv $arg_list\n";
               SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

               system("omthconv $arg_list") && SAS::fatal("Task Failure","omthconv failed.");
              

               #****************************************
               # Loop over all windows.
               #****************************************
               $win_list = ${${$exp_list}{$exposure_key}};


 
               foreach $win_key (keys(%{$win_list}))
               {
                  #*************************************
                  # Check consistancy of odf.
                  #
                  # AMK - 13/07/2000
                  #*************************************
                  if (exists(${$orb_key . $obs_key . $exp_type . $exp_key . $win_key}{IMI}))
                     {
                     #**************************
                     # Set pipeline variables.
                     #**************************
                     $image_filename = ${${$orb_key . $obs_key . $exp_type . $exp_key . $win_key}{"IMI"}};
                     $osw_list_intermediary_wavelet_filename = "osw_list_wavelet_" . $obs_key . "_" . $exp_type . $exp_key . substr($win_key, 1,1) . ".fit";

                     $number = substr($win_key,1,1)+1;
                     $flat_field_product_filename = "tmp_flat_field";

                     $modulo_8_product_filename = "tmp_mod8_file.fit";

                     #***********************************
                     # Need to start at 01 rather than 00
                     #***********************************
                     $sky_coord_pps_product_filename = "P" . $obs_key . "OM" . $exp_type . $exp_key . "SIMAGE" . $number . "000.FIT";
 

                     $image_pps_product_filename = "P" . $obs_key . "OM" . $exp_type . $exp_key . "IMAGE_" . $number . "000.FIT";
                     $osw_list_intermediary_detect_filename = "P" . $obs_key . "OM" . $exp_type . $exp_key . "SWSRLI" . $number . "000.FIT";

                     #************************************ 
                     # Rename region file as not a product
                     #************************************
                     $region_file = "tmp_region_file.asc";

                     #***********************************
                     # Produce image mode products.
                     #***********************************

                     $message = "... Running Image Mode Pipeline ...";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                     #*********************************
                     # omprep
                     #*********************************
                     $message = "... omprep...";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


                     $arg_list = "set=" . $inp_directory . '/' . $image_filename .
                                 " nphset=" . $inp_directory . '/' . $non_periodic_hk_filename .
                                 " pehset=" . $inp_directory . '/' . $periodic_hk_filename .
                                 " wdxset=" . $inp_directory . '/' . $window_data_filename .
                                 " outset=" . $out_directory . '/' . "tmp_image_1";

                     $message = "omprep $arg_list";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);
 

                     #**************************************************************
                     # Modified so that if omprep fails it just skips the observation,
                     # but does not kill the chain (1.11.2000)
                     #**************************************************************
                     system("omprep $arg_list");

                     $exit_value = $? >> 8;   # 0 okay, 1 call fatal called, or some other disaster
                     if ($exit_value eq 1) # Failure in omprep 
                     { 
                         $message = "omprep has detected an error- this observation will not be processed";
                         SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                     }     
                     else  # Observation okay
                     {

                     system("cp tmp_image_1 raw_imi.fit");
                
                     #******************************************
                     # omcosflag
                     #******************************************
                     $message = "... omcosflag ...";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                     $arg_list = "set=" . $out_directory . '/' . "tmp_image_1" .
                                 " thxset=" . $out_directory . '/' . "tmp_tracking";

                     $message = "omcosflag $arg_list";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                     system("omcosflag $arg_list") && SAS::fatal("Task Failure","omcosflag failed.");
                    

                    system("cp tmp_image_1 omcosflag.fit");
                    #****************************************
                    # omflatfield
                    #****************************************
                     $message = "... omflatfield ...";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                     $arg_list = "set=" . $out_directory . '/' . "tmp_image_1" .
                                 " thxset=" . $out_directory . '/' . "tmp_tracking" .
                                 " inorbitflatset="  . $in_orbit_flat_field_filename .
                                 " ppsflatset=" . $inp_directory . '/' . $flat_field_product_filename .
                                 " outset=" . $out_directory . '/' . "tmp_image_2";

                     $message = "omflatfield $arg_list\n";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


                     system("omflatfield $arg_list") && SAS::fatal("Task Failure","omflatfield failed.");
                   
                     system("cp tmp_image_2 omflatfield_imi.fit" . substr($win_key, 1,1));

                     #***************************************
                     # ommodmap
                     #***************************************
                     $message = "... ommodmap ...";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


                     $arg_list = "set=" . $out_directory . '/' . "tmp_image_2" .
                                 " ppsproduct=yes" .
                                 " ppsmod8set=" . $inp_directory . '/' . $modulo_8_product_filename .
                                 " outset=" . $out_directory . '/' . "tmp_image_1" .
                                 " nsig=5" .
                                 " nbox=16";  # Was originally 3

                     $message = "ommodmap $arg_list";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                     system("ommodmap $arg_list") && SAS::fatal("Task Failure","ommodmap failed.");
                     

                     system("cp tmp_image_1 ommodmappedimi.fit");
                     #**************************************************
                     # omdetect
                     #**************************************************
                     $message = "... omdetect ...";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


                     $arg_list = "set=" . $out_directory . '/' . "tmp_image_1" .
                                 " mod8set=" . $out_directory . '/' . $modulo_8_product_filename . 
                                 " flatset=" . $out_directory . '/' . $flat_field_product_filename .
                                 " outputregionfile=True" .
                                 " regionfile=" . $out_directory . '/' . $region_file .
                                 " outset=" . $out_directory . '/' . $osw_list_intermediary_detect_filename;

                     $message = "omdetect $arg_list";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


                     system("omdetect $arg_list") && SAS::fatal("Task Failure","omdetect failed.");


                     #*********************************************** 
                     # ommag
                     #***********************************************
                     $message = "... ommag ...";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


                     $arg_list = "set=" . $out_directory . '/' . $osw_list_intermediary_detect_filename .
                                 " wdxset=" . $inp_directory . '/' . $window_data_filename;

                     $message = "ommag $arg_list\n";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


                     system("ommag $arg_list") && SAS::fatal("Task Failure","ommag failed.");

                     #***********************************************
                     # omatt
                     #***********************************************
                     $message = "... omatt ...";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


                     $arg_list = "set=" . $out_directory . '/' . "tmp_image_1" .
                                 " sourcelistset=" . $out_directory . '/' . $osw_list_intermediary_detect_filename .
                                 " ppsoswset=" . $out_directory . '/' . $sky_coord_pps_product_filename.
                                 " usecat=F";

                     $message = "omatt $arg_list";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                     system("omatt $arg_list") && SAS::fatal("Task Failure","omatt failed.");


                     rename($out_directory . '/' . "tmp_image_1", $out_directory . '/' . $image_pps_product_filename);
                     unlink($out_directory . '/' . "tmp_image_2");
                     push(@osw_file_list, $out_directory . '/' . $osw_list_intermediary_detect_filename);


                 } #************ The end of if ($exit_value .eq. 1) construct  ***************
 


                     }
                  else
                     {

                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, "- ");

                     
                     $message = "Window '$win_key' of exposure '$exp_key' in observation '$obs_key' is missing an image file.";
                     SAS::warning("Task warning", $message);

                     $im_file = "RRRR_" . $obs_key . "_OM" . $exp_type . $exp_key . $win_key . "IMI.FIT";
		     $message = "****************************************";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                     $message = "Please insert file $im_file into the odf.";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

		     $message = "****************************************";
                     SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

                     }
                  }


                  unlink($out_directory . '/' . "tmp_tracking");
               }
  } #************ The end of if ($exit_value .eq. 1) construct  ***************
 
                  else
               {
               
               $message = "Exposure '$exp_key' in observation '$obs_key' is missing a file.";
               SAS::warning("Task warning", $message);

               $wdx_file = "RRRR_" . $obs_key . "_OM" . $exp_type .$exp_key . "00WDX.FIT";

               $message = "Please check file $wdx_file is in the odf.";
               SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

               }
            }

         #**********************************
         # Set remaining pipeline variables.
         #**********************************
         $observation_source_list_pps_product_filename = "P" . $obs_key . "OM" . $exp_type . "000" . "OBSRLS" . "0000.FIT";

         #**********************************************
         # omsrclistcomb.
         #********************************************** 
         $message = "... omsrclistcomb ...";
         SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

         $arg_list = "sourcelistsets=\"" . join(" ", @osw_file_list) . "\"" .
   
		     " nsigma=3" .    # nsigma was originally 1
                     " outset=" . $out_directory . '/' . $observation_source_list_pps_product_filename;

         $message = "omsrclistcomb $arg_list";
         SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


         system("omsrclistcomb $arg_list") && SAS::fatal("Task Failure","omsrclistcomb failed.");

         #**************************************************************************
         # ommosaic - Runs once for each different filter present in the observations
         #**************************************************************************          
         &RunOmmosaic;


#         }
#      else
#         {
#         print("*** Error ***\n");
#         print("Observation '$obs_key' is missing a flat field file.\n");
#         $ff_file = $orb_key . "_" . $obs_key . "_OMX00000FFX.FIT";
#         print("Please insert file $ff_file into the odf.\n");
#         }

      }


   }




   $date=&date_time;
   $message = "Finished running SAS task $taskname V$VERSION $date";
   SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

#   print($create, "\n");

#////////////////////////////////////////////////
# AMk - close the log file
#   close(STDOUT);
#   close(STDERR);
#///////////////////////////////////////////////
}
#
#=========================================================================
#
# subroutine section
#
#=========================================================================
sub date_time {          # create the file creation datestring

    local $datestring;
    $datestring = `date -u`;
    chop $datestring;
    return $datestring;
}
#********************************************************************
# subroutine RunOmmosaic - 23.01.2000
# Produces a list of the SIMAGE2000 files for each filter present
# in the ODF data set, which are then fed into omosaic to produce 
# a mosaiced image.
#********************************************************************
sub RunOmmosaic
{
    @SASFiles = <$inp_directory*.SAS>;  #  inp_directory added 1st March 2001        
 
    $SASFile = shift @SASFiles;


    open (SASFILE, $SASFile);

    @FilterList;               # Stores a list of the filters in the summary file
    @ExposureNoList;           # Stores a list of the exposure numbers
    @ExposureFilterList;       # Stores a list of the filter for each exposure number

 
    #*************************************************
    # Skip past the EPIC and RGS stuff
    #*************************************************

    $line = <SASFILE>;
       
    until (substr($line, 0, 2) eq "OM")
    {
        $line = <SASFILE>;
                                  
    } 

  
    while ($line = <SASFILE>)
    {
        
        if ($line =~ "/ Exposure Identifier")
        {        
            $exposureNo = substr($line, 0, 4);
            push(@ExposureNoList, $exposureNo);         
            $line = <SASFILE>;
            until($line =~ "FILTER =")
            {
	        $line = <SASFILE>;
           
            } 
            
            $pos = index($line, "//");

            $filterName = substr($line, 9, $pos - 10); # Get rid of the characters after the end of the filter
            if ($filterName eq "White")
            {
                $filterName = "WHITE";
	    }

           
            push(@ExposureFilterList, $filterName);

            #*********************************************
            # Add the filter to the list
            #*********************************************
            $numFilters = @FilterList;
            $exists = 0;
       
        
            for ($i = 0; $i <  $numFilters ; $i++)
            {
	   
                if ($filterName eq @FilterList[$i])
                {
               
                    $exists = 1;
                    break;

                }

            }

            if ($exists eq 0)
            {
           
                push(@FilterList, $filterName);
	    }
        }
    }
    close SASFILE;



    @Simage2000Files = <*SIMAGE2000*> ;    # Added on 1st March 2001    
 

    $numFilters = @FilterList;
    $numExposures = @ExposureNoList;
  
   
    $numFiles = @Simage2000Files;
    

    #******************************************************
    # Compose a list of the SIMAGE2000 files for each filter
    # and then run ommosaic
    #******************************************************
    
    for ($k = 0; $k < $numFilters; $k++)
    {    

        @osw_simage_list =();           # Initialise the file list    
        for ($j = 0; $j < $numFiles; $j++)
        {
               
            $text = @Simage2000Files[$j];       # The SIMAGE2000 file name
            $text1 = substr($text, 14, 3);      # The exposure number

            for ($i = 0; $i <  $numExposures; $i++)
            {
                if (@ExposureFilterList[$i] eq @FilterList[$k])
                { 
                    $text2 = substr(@ExposureNoList[$i], 0, 3);
            
                    if ($text1 eq $text2)  
                    {                      
                        push(@osw_simage_list, $text);  # Add the file name to the list    
                    }            
                }
            }
	}
               
        #************************************************
        # ommosaic
        #************************************************          
        $message = "... ommosaic- mosaicing images for filter ". @FilterList[$k]."\n";
        SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

        $temp= @osw_simage_list;
        $outputFileName = substr(@osw_simage_list[0], 0, 14) . &GetFilterCode(@FilterList[$k]) . "SIMAGE2000.FIT";
       	 

        if ($temp >  0) 
        {
            $arg_list = "imagesets=\"" . join(" ", @osw_simage_list) . "\" mosaicedset=". $out_directory . $outputFileName;


            $message = "ommosaic $arg_list";
            SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

            system("ommosaic $arg_list") && SAS::fatal("Task Failure", "ommosaic");
        }
        else
        {
            $message = "No image files to run ommosaic!";
            SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);


        }
         
  
    }
}

#**************************************
# subroutine GetFilterCode
# Given a filter from the SAS file
# returns the filter code to be used
# in constructing the output file name.
#**************************************
sub GetFilterCode
{
    my($filter) = @_;  # The input filter name
    my($name);
    
    @OMFilters = ("BLOCKED", "V",  "MAGNIFIER", "U",  "B",  "WHITE", "UVW1", "UVM2", "UVW2");
    @OMFilterCodes = ("BL",  "VB", "MA",        "UB", "BB", "WL",    "W1",   "M2",   "W2");

    for ($i = 0; $i < 9; $i++)
    {
       
        if (@OMFilters[$i] eq $filter)
        {
            $name = @OMFilterCodes[$i];
            return $name;
	}
    }
   
    
}
#*****************************************************
# FindFile - Searches the directory for the file $
#
#*****************************************************
sub FindFile($)
{
    my ( $fileName) = @_;
    local $number = 0;

    foreach $file (@filenames)
    {
        if ($file eq $fileName) 
        {
            $number = $number + 1;
        } 
    }
    if ($number eq 0)
    {
        $message = "File not found";
        SAS::message($SAS::AppMsg, $SAS::SparseMsg, $message);

    }
    return $number;    

}



















































































































































